#include <ctype.h>
#include <string.h>
#include "automaton.h"
#include "basicTypedefs.h"
#include "global.h"
#include "suggBasics.h"


/* Things made global to save stack during the recursion. */
/* Length of the requested pattern. */
unsigned char pattern_length ;
/* Current requested pattern. */
unsigned char *pattern ;
/* Buffer where to store cheat suggestions. */
unsigned char prefix[MAX_STRINGLEN] ;




void recFind (unsigned long base, unsigned char pattern_index, int prefix_index,
              BoolTy was_final)
{
  /* Need to be int otherwise char is not wide range enought for the <= test */
  unsigned int letter_code, start_letter, stop_letter ;
  unsigned int turn_step = 1 ;
  unsigned int pattern_increment = 1 ;

  if (pattern_index == pattern_length) {
    /* We reached the end of the pattern, so now we have to check if the prefix
       accumulated up to now is a valid word in our dictionary. */
    if (was_final) {
      /* The found solution is a complete word of the dictionary. */
      prefix[prefix_index] = '\0' ;
      /* Insert the suggestion in the list.
         Note: AddSuggestion MUST copy the string passed in argument ! */
      addSuggestion (prefix, 0) ;
    }        /* End of if ((base != END_TRANSITION) && */
  }          /* End of if (pattern_index == pattern_length) */
  else {
    /* First if we are on the terminal transition of the automaton, we must
       leave.
       This means that the pattern is longer than any word we can find with
       the current prefix accumulated along the path in the DAG we followed. */
    if (base == END_TRANSITION) return ;
    /* We did not reach the end of the pattern, so let go on further to
       accumulate letters in the prefix.
       We have several cases to investigate. Pattern letter is '?' '*' or
       other, and case in significant or not. */
    switch (pattern[pattern_index]) {
    case '?':
      pattern_increment = 1 ;
      start_letter = 1 ;
      stop_letter = 255 ;
      break ;                           /* End of '?' */

    case '*':
      recFind (base, pattern_index + 1, prefix_index, was_final) ;
      /* False end of '*' !!! False through !!!! */
      pattern_increment = 0 ;
      start_letter = 1 ;
      stop_letter = 255 ;
      break ;                           /* End of '*' */

    default:
      pattern_increment = 1 ;
      if (case_sensitive) {
        start_letter = (unsigned int) pattern[pattern_index] ;
        stop_letter = start_letter ;
      }                                 /* End of if (case_sensitive) */
      else {
        // Determine le nombre de tours qu'il faudra faire en fonction
        // de l'importance de la casse. Ce nombre de tour est gouverne par
        // le pas d'incrementation de la boucle for qui va suivre.
        // Note: ASCII majuscule < ASCII minuscule.
        start_letter = (unsigned int) toupper (pattern[pattern_index]) ;
        stop_letter = (unsigned int) tolower (pattern[pattern_index]) ;
        // Attention, si les lettres de depart sont identiques apres
        // differenciation de la casse (par exemple, certaines lettres
        // n'ont pas a proprement parler de notion de majuscule-minuscule,
        // alors, il fait faire en sorte que turn_step ne soit pas egal
        // a 0, sinon la boucle ci-dessous ne finira jamais !!!
        if (start_letter == stop_letter) turn_step = 1 ;
        else turn_step = stop_letter - start_letter ;
      }            /* End of else of if (case_sensitive) */
      break ;      /* End of default case for switch (pattern[pattern_index]) */
    }              /* End of switch (pattern[pattern_index]) */

    /* Generic, now ! Let's work ! */
    for (letter_code = start_letter; letter_code <= stop_letter;
         letter_code+=turn_step) {
      if ((base + letter_code) < automaton_size) {
        /* We check if there exists a transition. */
        if ((GETCHAR (automaton[base + letter_code])) == letter_code) {
          prefix[prefix_index] = letter_code ;
          recFind (GETNEXTBASE (automaton[base + letter_code]),
                   pattern_index + pattern_increment, prefix_index + 1,
                   (BoolTy) GETISFINAL (automaton[base + letter_code])) ;
        }    /* End of
                if ((GETCHAR (automaton[base + letter_code])) == letter_code) */
      }      /* End of if ((base + letter_code) < automaton_size) */
    }        /* End of for (letter_code = start_letter;... */
  }          /* End of else of if (pattern_index == pattern_length) */
}




void cheat (unsigned char *patt)
{
  /* Just to make them global and save a bit a stack during recursion. */
  unsigned char expanded_pattern[MAX_STRINGLEN] ;
  int i,j ;

  /* Expand the pattern (to remove '+'). */
  for (i = 0, j = 0; i < (int) strlen ((char*) patt); i++) {
    switch (patt[i]) {
    case '+':
      /* Expand the usual '+' wildcard into '?*'. */
      expanded_pattern[j] = '?' ;
      j++ ;
      expanded_pattern[j] = '*' ;
      j++ ;
      break ;                 // End of '?'

    default:
      expanded_pattern[j] = patt[i] ;
      j++ ;
      break ;                 /* End of default case of switch (patt[i]) */
    }                         /* End of switch (patt[i]) */
  }                           /* End of for (i=0; i<strlen(patt); i++) */
  expanded_pattern[j] = '\0' ;

  pattern = expanded_pattern ;
  pattern_length = strlen ((char*) pattern) ;
  recFind (initial_state, 0, 0, FALSE) ;
}
