#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "suggBasics.h"


struct suggestion *suggestions = NULL ;
int number_suggestions = 0 ;



/* Note: addSuggestion MUST copy the string passed in argument ! */
void addSuggestion (unsigned char *word, int weight)
{
  struct suggestion *tmp ;

  /* This functiom must copy the string passed in argument !
     First, it searches if the word already exists as a sugggestion.
     If not, then it add it with the given weight.
     If yes, then if the given weight is lower than the weight already
     associated to the suggestion, then we update the weight, otherwise
     one do nothing. */
  tmp = suggestions ;
  while (tmp != NULL) {
    if (strcmp ((char*) tmp->string, (char*) word) == 0) {
      // Just update the weight of the suggestion
      if (weight < tmp->weight) tmp->weight = weight ;
      return ;
    }
    tmp = tmp->next ;
  }

  /* If we get here, that's because the word do not exist yet as a
     suggestion. */
  tmp = (struct suggestion*) malloc (sizeof (struct suggestion)) ;
  tmp->string = (unsigned char*)
    malloc ((1 + strlen ((char*) word)) * sizeof (unsigned char)) ;
  strcpy ((char*) tmp->string, (char*) word) ;
  tmp->weight = weight ;
  /* Insert in head. */
  tmp->next = suggestions ;
  number_suggestions++ ;
  suggestions = tmp ;
}



void releaseSuggestions ()
{
  struct suggestion *tmp0, *tmp1 ;

  tmp0 = suggestions ;
  while (tmp0 != NULL) {
    free (tmp0->string) ;
    tmp1 = tmp0->next ;
    free (tmp0) ;
    tmp0 = tmp1 ;
  }
  suggestions = NULL ;
  number_suggestions = 0 ;
}




void displaySuggestions ()
{
  struct suggestion *tmp ;
  struct suggestion **sugg_table ;
  int inserted_count = 0 ;
  int i, j ;
  int space_needed = 0 ;
  char *all_suggestions_buffer ;

  /* Directly skip everything os no suggestion, so that's it... */
  if (suggestions == NULL) return ;

  sugg_table =
    (struct suggestion**)
    malloc (number_suggestions * sizeof (struct suggestion*)) ;

  tmp = suggestions ;
  while (tmp != NULL) {
    /* Insert the string of this suggestion in the table, sorted first by
       error weight, then alphabetically.
       First, sort by increasing weight and stop as soon as we found
       a suggestion with a weight equal or greater. */
    for (i = 0; i<inserted_count; i++) {
      if (sugg_table[i]->weight >= tmp->weight) break ;
    }
    /* Then, sort alphabetically amongst suggestions with this weight. */
    for (;i<inserted_count; i++) {
      if ((sugg_table[i]->weight > tmp->weight)
          || (strcmp ((char*) sugg_table[i]->string, (char*) tmp->string) >= 0))
        break ;
    }
    /* Here, i equals the index where to insert the new suggestion.
       Everything beyond must be shifted one case farther. */
    for (j=inserted_count-1; j>=i; j--) sugg_table[j+1] = sugg_table[j] ;
    sugg_table[i] = tmp ;
    tmp = tmp->next ;
    inserted_count++ ;
    /* Add 1 for the "\n". */
    space_needed += 1 + strlen ((char *) sugg_table[i]->string) ;
  }

  /* Assertion test. Must always held ! Indeed, just some debug.
  if (number_suggestions != inserted_count)
  ASSERT_FAILED ; */

  /* Allocate the big text buffer to contain all the suggestions. */
  all_suggestions_buffer = (char*) malloc ((1 + space_needed) * sizeof (char)) ;
  j = 0 ;
  /* Then insert the sorted strings contained in the table in this buffer. */
  for (i = 0; i< number_suggestions; i++) {
    sprintf (&all_suggestions_buffer[j], "%s\n", (char*) sugg_table[i]->string) ;
    /* Add 1 for the "r\n". */
    j = 1 + j + strlen ((char*) sugg_table[i]->string) ;
  }

  /* Finally insert the big text in the standard output. */
  printf ("%s", all_suggestions_buffer) ;

  /* Release the memory and the whole suggestions big text buffer.
     We don't release the global suggestion here to avoid a partial
     desallocation in case where this thread is kill prematurately. */
  free (sugg_table) ;
  free (all_suggestions_buffer) ;
}



char* getStringFromSuggestions ()
{
  struct suggestion *tmp ;
  struct suggestion **sugg_table ;
  int inserted_count = 0 ;
  int i, j ;
  int space_needed = 0 ;
  char *all_suggestions_buffer ;

  /* Directly skip everything os no suggestion, so that's it... */
  if (suggestions == NULL) return (NULL) ;

  sugg_table =
    (struct suggestion**)
    malloc (number_suggestions * sizeof (struct suggestion*)) ;

  tmp = suggestions ;
  while (tmp != NULL) {
    /* Insert the string of this suggestion in the table, sorted first by
       error weight, then alphabetically.
       First, sort by increasing weight and stop as soon as we found
       a suggestion with a weight equal or greater. */
    for (i = 0; i<inserted_count; i++) {
      if (sugg_table[i]->weight >= tmp->weight) break ;
    }
    /* Then, sort alphabetically amongst suggestions with this weight. */
    for (;i<inserted_count; i++) {
      if ((sugg_table[i]->weight > tmp->weight)
          || (strcmp ((char*) sugg_table[i]->string, (char*) tmp->string) >= 0))
        break ;
    }
    /* Here, i equals the index where to insert the new suggestion.
       Everything beyond must be shifted one case farther. */
    for (j=inserted_count-1; j>=i; j--) sugg_table[j+1] = sugg_table[j] ;
    sugg_table[i] = tmp ;
    tmp = tmp->next ;
    inserted_count++ ;
    /* Add 1 for the "\n". */
    space_needed += 1 + strlen ((char *) sugg_table[i]->string) ;
  }

  /* Assertion test. Must always held ! Indeed, just some debug.
  if (number_suggestions != inserted_count)
  ASSERT_FAILED ; */

  /* Allocate the big text buffer to contain all the suggestions. */
  all_suggestions_buffer = (char*) malloc ((1 + space_needed) * sizeof (char)) ;
  j = 0 ;
  /* Then insert the sorted strings contained in the table in this buffer. */
  for (i = 0; i< number_suggestions; i++) {
    sprintf (&all_suggestions_buffer[j], "%s\n", (char*) sugg_table[i]->string) ;
    /* Add 1 for the "r\n". */
    j = 1 + j + strlen ((char*) sugg_table[i]->string) ;
  }

  /* Release the memory of the sorted suggestions. */
  free (sugg_table) ;
  /* Do not release the big string to be able to access it later because
     we return a pointer on it. */
  return (all_suggestions_buffer) ;
}
