#include <string.h>
#include <ctype.h>
#include "basicTypedefs.h"
#include "automaton.h"
#include "global.h"
#include "suggBasics.h"

enum operation { Nop, Suppr, Add, Wrong } ;

int num_allowed_errors ;
unsigned char sugg_word[MAX_STRINGLEN] ;
int len_wword ;
unsigned char *wrong_word ;




void recSuggest (unsigned long base, enum operation last_op, int index_wword,
                 int sugg_word_index, BoolTy was_final, int err_cnt)
{
  int real_err_cnt ;
  unsigned char good_letter_code ;
  unsigned int letter_code ;
  DAGElement new_bucket ;
  unsigned long new_base ;
  BoolTy is_final ;

  if (err_cnt <= num_allowed_errors) {
    if ((base == END_TRANSITION) || (was_final)) {
      /* The real number of errors must take into account the difference of
         length between the original word and the place where we are currently
         inside.
         If, according to this place, there still is 2 letters in the original
         word, this corresponds in fact to a found suggestion with 2 "forgets"
         in addition to the already accumulated errors. Be careful the current
         index refers a letter we don't have yet processed, so the real number
         of errors is:
             (len_wword - 1 - index_wword + err_cnt) + 1
         We shorten by removing - 1 + 1. */
      real_err_cnt = len_wword - index_wword + err_cnt ;
      if (real_err_cnt <= num_allowed_errors) {
        sugg_word[sugg_word_index] = '\0' ;
        /* Note: AddSuggestion MUST copy the string passed in argument ! */
        addSuggestion (sugg_word, real_err_cnt) ;
      }
    }
    
    if (base != END_TRANSITION) {
      if (index_wword < len_wword) {
        /* Let's consider that the current letter is correct and then that the
           current letter is wrong (factorise). */
        good_letter_code = (unsigned int) wrong_word[index_wword] ;
        for (letter_code = 1; letter_code <= 255; letter_code++) {
          if (((base + letter_code) < automaton_size) &&
              ((unsigned int) GETCHAR (automaton[base + letter_code]) ==
               letter_code)) {
            /* Common sub-expression sharing. */
            new_bucket = automaton[base + letter_code] ;
            new_base = GETNEXTBASE (new_bucket) ;
            is_final = GETISFINAL (new_bucket) ;
            /* We just change the weight and the last performed operation
               according to if we are in the case where the letter is correct
               or wrong and according to if we are case sensitive or not. */
            sugg_word[sugg_word_index] = (unsigned char) letter_code ;
            if ((letter_code == ((unsigned int) good_letter_code)) ||
                ((!case_sensitive) &&
                 (tolower (letter_code) == tolower (good_letter_code))))
              recSuggest (new_base, Nop, (index_wword + 1),
                          (sugg_word_index + 1), is_final, err_cnt) ;
            else
              recSuggest (new_base, Wrong, (index_wword + 1),
                          (sugg_word_index + 1), is_final, (err_cnt + 1)) ;
          }
        }
        
        /* Let's consider that there is an extra letter in the word. */
        if (last_op != Add)
          recSuggest (base, Suppr, (index_wword + 1), sugg_word_index,
                      FALSE, (err_cnt + 1)) ;

        /* Let's consider that there is a missing letter in the word. */
        if (last_op != Suppr) {
          for (letter_code = 1; letter_code <= 255; letter_code++) {
            if (((base + letter_code) < automaton_size) &&
                ((unsigned int) GETCHAR (automaton[base + letter_code]) ==
                 letter_code)) {
              /* Common sub-expression sharing. */
              new_bucket = automaton[base + letter_code] ;
              new_base = GETNEXTBASE (new_bucket) ;
              is_final = GETISFINAL (new_bucket) ;
              sugg_word[sugg_word_index] = (unsigned char) letter_code ;
              recSuggest (new_base, Add, index_wword, (sugg_word_index + 1),
                          is_final, (err_cnt + 1)) ;
            }
          }
        }
      }
    }
  }
}



void suggest (unsigned char *word)
{
  /* Make some stuff global to save memory on the stack during recursion. */
  len_wword = strlen ((char*) word) ;
  wrong_word = word ;
  if (len_wword >= 7) num_allowed_errors = 2 ;
  else num_allowed_errors = 1 ;
  recSuggest (initial_state, Nop, 0, 0, FALSE, 0) ;
}
