//
//  IDicTouchAppDelegate.m
//  IDic
//
//  Created by Didou on 08/09/09.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//
#import "GlobalITouchData.h"
#import "IDicTouchAppDelegate.h"
#include "global.h"
#include "automaton.h"
#include "dicManagement.h"



@implementation IDicTouchAppDelegate
@synthesize window ;
@synthesize tabBarController ;



/* **************************************************
   Kind: System.
   Override point for customization after app launch.
   ************************************************** */
- (void) applicationDidFinishLaunching: (UIApplication *) application
{
  /* Handle the preferences. */
  [IDicTouchAppDelegate initializePrefs] ;
  [IDicTouchAppDelegate loadPrefsAndApply] ;
  /* Add the tab bar controller's current view as a subview of the window. */
  [window addSubview: tabBarController.view] ;
}



/*
// Optional UITabBarControllerDelegate method
- (void) tabBarController:
            (UITabBarController *) tabBarController
            didSelectViewController: (UIViewController *) viewController
{
}
*/



/*
// Optional UITabBarControllerDelegate method
- (void) tabBarController:
           (UITabBarController*) tabBarController
           didEndCustomizingViewControllers: (NSArray*) viewControllers
           changed: (BOOL) changed
{
}
*/



/* *************
   Kind: System.
   ************* */
- (void) applicationWillTerminate: (UIApplication *) application
{
  /* Dictionary memory management to free allocated memory. */
  unloadDictionary () ;
}



/* *************
   Kind: System.
   ************* */
- (void) dealloc
{
  [tabBarController release] ;
  [window release] ;
  [super dealloc] ;
}



/* Consider incrementing the 2 last digits each time the preferences format
   is no mode compatible with the one of the previous version. */
#define PREFS_MAGIC_NUMBER TAGTOLONG('I','P','0','0')

/* *******************
   Kind: Not exported.
   ******************* */
+ (BOOL) initializePrefs
{
  NSUserDefaults *prefsDb = [NSUserDefaults standardUserDefaults] ;
  NSData *testIfPrefSaved = [prefsDb objectForKey: prefMagicNumberKey] ;
  BOOL saveStatus ;
	
  /* Verifies if some preferences were saved. */
  if (testIfPrefSaved != nil) {
    NSInteger magicNumber = [prefsDb integerForKey: prefMagicNumberKey] ;
    if (magicNumber == PREFS_MAGIC_NUMBER) return (YES) ;
  }
	
  /* Either there were not saved preferences or some but with a wrong magic
     number. Then creare some fresh and default ones. */
	NSNumber *magicNumber = [NSNumber numberWithInt: PREFS_MAGIC_NUMBER] ;
  NSNumber *preferedDicNameIndex = [NSNumber numberWithInt: 0] ;
  NSNumber *preferedCaseSensitivity = [NSNumber numberWithBool: NO] ;
  NSDictionary *idictouchAppPrefs =
    [[NSDictionary alloc]
      initWithObjectsAndKeys:
		    magicNumber, prefMagicNumberKey,
        preferedDicNameIndex, prefDicNameIndexKey,
        preferedCaseSensitivity, prefCaseSensitiveKey,
      nil] ;
  [prefsDb registerDefaults: idictouchAppPrefs] ;
  if ([prefsDb synchronize]) saveStatus = YES ;
	else {
    /* Synchronisation failed... */
		saveStatus = NO ;
	}
	
	[idictouchAppPrefs release] ;
  return (saveStatus) ;
}



/* *******************
   Kind: Not exported.
   ******************* */
+ (void) loadPrefsAndApply
{
  BOOL status ;
  NSUserDefaults *prefsDb = [NSUserDefaults standardUserDefaults] ;

  NSInteger preferedDicNameIndex =
    [prefsDb integerForKey: prefDicNameIndexKey] ;
  BOOL preferedCaseSensitivity = [prefsDb boolForKey: prefCaseSensitiveKey] ;

  /*
  char txtAsBytes[256] ;
  [preferedDicName getCString:
    txtAsBytes maxLength: 256 encoding: NSISOLatin1StringEncoding] ;
  printf ("Dic:%s Sensitive: %d\n", txtAsBytes, preferedCaseSensitivity) ;
  */

  /* Dictionary loading. */
  currentDicNameIndex = (int) preferedDicNameIndex ;
  status =
    [IDicTouchAppDelegate
     initDictionary: availDictsNames [preferedDicNameIndex]] ;
  if (! status) {
    UIAlertView *fatalAlert =
      [[UIAlertView alloc] 
        initWithTitle: @"Fatal error"
        message: NSLocalizedString (@"strDicLoadFailed", @"Dic loading failed")
        delegate: self cancelButtonTitle: @"Ok" otherButtonTitles: nil] ;
    [fatalAlert show] ;
    [fatalAlert release] ;
  }
  /* Case sensitivity. */
  if (preferedCaseSensitivity) case_sensitive = TRUE ;
  else case_sensitive = FALSE ;
}



/* ***************
   Kind: Exported.
   *************** */
+ (BOOL) savePrefs
{
  BOOL sensitivityToSave ;
  NSUserDefaults *prefsDb = [NSUserDefaults standardUserDefaults] ;

	/* Always save the magic number. */
	[prefsDb
	  setInteger: (NSInteger) PREFS_MAGIC_NUMBER forKey: prefMagicNumberKey] ;
  /* Dictionary saving. */
  [prefsDb
    setInteger: (NSInteger) currentDicNameIndex
    forKey: prefDicNameIndexKey] ;
  /* Case sensitivity. */
  if (case_sensitive) sensitivityToSave = YES ;
  else sensitivityToSave = NO ;
  [prefsDb setBool: sensitivityToSave forKey: prefCaseSensitiveKey] ;

  return ([prefsDb synchronize]) ;
}



/* ***************
   Kind: Exported.
   *************** */
+ (BOOL) initDictionary: (NSString*) dicBaseName
{
  unsigned long *rawData ;
  BOOL success ;

  NSString *dicPath =
    [[NSBundle mainBundle] pathForResource: dicBaseName ofType: @"bin"] ;
  NSData *dicData = [[NSData alloc] initWithContentsOfFile: dicPath] ;
  rawData = (unsigned long*) [dicData bytes] ;
  success = loadDictionaryFromRawData (rawData) ;
  [dicData release] ;
  return (success) ;
}

@end
