//
//  OptionsViewController.m
//  IDic
//
//  Created by Didou on 08/09/09.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//
#import "globalITouchData.h"
#import "IDicTouchAppDelegate.h"
#import "OptionsViewController.h"
#include "basicTypedefs.h"
#include "global.h"
#include "dicManagement.h"

@implementation OptionsViewController

@synthesize sensitiveSwitch ;
@synthesize dictionaryPicker ;
@synthesize resetButton ;
@synthesize dicNamesForPickerArray ;

/*
 // The designated initializer. Override to perform setup that is required before the view is loaded.
 - (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
 if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
 // Custom initialization
 }
 return self;
 }
 */

/*
 // Implement loadView to create a view hierarchy programmatically, without using a nib.
 - (void)loadView {
 }
 */


- (void) viewDidLoad
{
  [super viewDidLoad] ;

  /* The array containing the names of the available dictionaries. Directly
     initialise it with our C-like array content. */
  dicNamesForPickerArray =
    [[[NSArray alloc]
      initWithObjects: availDictsNames count: NUM_AVAILABLE_DICTS] retain] ;

  dictionaryPicker.autoresizingMask = UIViewAutoresizingFlexibleWidth ;
  /* Note that default is NO. */
  dictionaryPicker.showsSelectionIndicator = YES ;

  /* This view controller is the data source and delegate. Don't forget to
     make the links under IBuilder. */
  dictionaryPicker.delegate = self ;
  dictionaryPicker.dataSource = self ;

  /* Add this picker to our view controller and make it visible. */
  dictionaryPicker.hidden = NO ;
  [self.view addSubview: dictionaryPicker] ;
}



/*
 // Override to allow orientations other than the default portrait orientation.
 - (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
 // Return YES for supported orientations
 return (interfaceOrientation == UIInterfaceOrientationPortrait);
 }
 */

- (void) didReceiveMemoryWarning
{
  // Releases the view if it doesn't have a superview.
  [super didReceiveMemoryWarning] ;

  // Release any cached data, images, etc that aren't in use.
}



- (void) viewDidUnload
{
  // Release any retained subviews of the main view.
  // e.g. self.myOutlet = nil;
}



- (void) setOptionIHMWithCurrentSettings
{
  BOOL switchVal ;

  /* Select the case sensitivity. */
  if (case_sensitive) switchVal = YES ;
  else switchVal = NO ;
  [sensitiveSwitch setOn: switchVal animated: NO] ;
  /* Select the dictionary name according to the current index. */
  [dictionaryPicker
   selectRow: currentDicNameIndex inComponent: 0 animated: NO] ;
}



- (void) viewDidAppear: (BOOL) animated
{
  [super viewDidAppear: animated] ;
  /* Initialise the IHM with the current prefs. */
  [self setOptionIHMWithCurrentSettings] ;
}



/* Called before the view is dismissed, covered, or otherwise hidden from
   view. */
- (void) viewWillDisappear: (BOOL) animated
{
  /* Apply the new prefs. */
  if (sensitiveSwitch.on) case_sensitive = TRUE ;
  else case_sensitive = FALSE ;
  // TODO Dico currentDicNameIndex Ancienne valeur:
  NSInteger newDicNameIndex = [dictionaryPicker selectedRowInComponent: 0] ;
  if (newDicNameIndex != currentDicNameIndex) {
    /* The selected dictionary is different of thr repviously used.
       We set the index of the newly used dictionary to be saved. */
    currentDicNameIndex = newDicNameIndex ;
    /* Release the previous dictionary. */
    unloadDictionary () ;
    /* Load the new ditionary. */
    [IDicTouchAppDelegate
      initDictionary: availDictsNames [currentDicNameIndex]] ;
		/* Updating the abbreviation indicating the currently used dictionary will
			 be done by the "UseView" when it is display again. So, nothing to do
			 here. */
  }
  /* Finally, always save the preferences. */
  [IDicTouchAppDelegate savePrefs] ;
  [super viewDidDisappear: animated] ;
}



- (void) dealloc
{
  [dicNamesForPickerArray release] ;
  [sensitiveSwitch dealloc] ;
  [dictionaryPicker dealloc] ;
  [resetButton dealloc] ;
  [super dealloc] ;
}



- (IBAction) resetButtonClicked: (id) sender
{
  [self setOptionIHMWithCurrentSettings] ;
}



#pragma mark -
#pragma mark UIPickerViewDataSource
- (NSInteger) numberOfComponentsInPickerView: (UIPickerView*) pickerView
{
  return (1) ; /* This makes our picker having 1 slot. */
}



- (NSInteger) pickerView: (UIPickerView*) pickerView
 numberOfRowsInComponent: (NSInteger) component
{
  return [dicNamesForPickerArray count] ;
}



/* This method is responsible in redering the picker's cells in the view. */
- (NSString*) pickerView: (UIPickerView*) pickerView
             titleForRow: (NSInteger) row
            forComponent: (NSInteger) component
{
  NSString *returnStr = @"" ;
  if (pickerView == dictionaryPicker) {
    returnStr = [dicNamesForPickerArray objectAtIndex: row] ;
  }
  return (returnStr) ;
}



#pragma mark -
#pragma mark UIPickerViewDelegate
- (void) pickerView: (UIPickerView*) pickerView
       didSelectRow: (NSInteger) row
        inComponent: (NSInteger) component
{
  if (pickerView == dictionaryPicker) {
    /* Report the selection to the console. */
    NSString *dName =
      [NSString
        stringWithFormat: @"%@",
        [dicNamesForPickerArray
          objectAtIndex: [pickerView selectedRowInComponent: 0]]] ;
    NSLog (dName) ;
  }
}

@end
